/*******************************************************************************
*                                                                              *
* System      : SOIPIX readout module                                          *
* Block       : Example for the User FPGA                                      *
* Module      : SOIPIX_USER                                                    *
* Version     : v 0.0.0 2008/05/19                                             *
*                                                                              *
* Description : Top Module                                                     *
*                                                                              *
* Designer    : Tomohisa Uchida                                                *
*                                                                              *
*                Copyright (c) 2008 Tomohisa Uchida  & Yasuyuki Horii!         *
*                All rights reserved                                           *
*                                                                              *
*******************************************************************************/
`define FPGA_VER 32'h0805_1901
`define FPGA_ID  32'h0005_0000

//`include "../Src/sys_dcm.v"
//`include "../Src/SIO_SLAVE.v"
//`include "../Src/SIO_REG.v"

module SOIPIX_USER(
// System
	OSC					,	// in	: 50MHz clock
	SW_RSTn				,	// in	: System reset
// User I/F
	USR_CLK				,	// out	: Clock
	USR_ACTIVE			,	// in	: TCP established
	USR_CLOSE_REQ		,	// in	: Request to close a TCP Connection
	USR_CLOSE_ACK		,	// out	: Acknowledge for USR_CLOSE_REQ
	// TCP Tx
	USR_TX_AFULL		,	// in	: Almost full flag of a TCP Tx FIFO
	USR_TX_WE			,	// out	: TCP Tx Data write enable
	USR_TX_WD			,	// out	: TCP Tx Data[7:0]
	// TCP Rx
	USR_RX_EMPTY		,	// in	: Empty flag of a TCP RX flag
	USR_RX_RE			,	// out	: TCP Rx Data Read enable
	USR_RX_RV			,	// in	: TCP Rx data valid
	USR_RX_RD			,	// in	: TCP Rx data[7:0]
	// Register Access
	REG_CLK				,	// in	: Clock
	REG_ACT				,	// in	: Access Active
	REG_DI				,	// out	: Serial Data Output
	REG_DO				,	// in	: Serial Data Input
	// SBPIX
	CSR,
	RA,
	CA,
	RAEN,
	CAEN,
	WR,
	CLK,
	TRIG,
	DD,
	DO,
	DI,
// SW, LED
	LED					,	// out	: LED[7:0]
	DIP_SW					// in	: DIP SW[7:0]
);

//-------- Input/Output -------------
	input			OSC				;
	input			SW_RSTn			;

	output		USR_CLK			;
	input			USR_ACTIVE		;
	input			USR_CLOSE_REQ	;
	output		USR_CLOSE_ACK	;
	input			USR_TX_AFULL	;
	output		USR_TX_WE		;
	output		[7:0]	USR_TX_WD;
	input			USR_RX_EMPTY	;
	output		USR_RX_RE		;
	input			USR_RX_RV		;
	input			[7:0]	USR_RX_RD;
	input			REG_CLK			;
	input			REG_ACT			;
	output		REG_DI			;
	input			REG_DO			;

	output	CSR			;
	output 	[5:0] RA		;
	output 	[5:0] CA		;
	output 	RAEN			;
	output 	CAEN			;
	output 	WR				;
	output 	CLK			;
	output 	TRIG			;
	input 	[15:0] DD	;
	input 	[4:0] DO		;
	output 	[4:0] DI		;

	output	[7:0]	LED	;
	input		[7:0]	DIP_SW;

//------------------------------------------------------------------------------
//	DLL
//------------------------------------------------------------------------------
	wire			RST_DLL				;
	wire			locked				;
	wire			CLK25M				;
	wire			CLK50M				;

	assign	RST_DLL		= ~SW_RSTn;

   	sys_dcm			SYSDCM(
		.CLKIN_IN			(OSC			),
		.RST_IN				(RST_DLL		),
		.CLKDV_OUT			(CLK25M			),
		.CLKIN_IBUFG_OUT	(),
		.CLK0_OUT			(CLK50M			),
		.LOCKED_OUT			(locked			)
	);

	assign	RSTn	= locked & ~RST_DLL;

	ODDR	IOB_DDRCLKp0(.Q(USR_CLK), .C(CLK25M), .CE(1'b1), .D1(1'b0), .D2(1'b1), .R( 1'b0), .S( 1'b0));

//------------------------------------------------------------------------------
//	Contents of Reg => LED
//------------------------------------------------------------------------------
	wire	[7:0]	LED			;
	reg	[7:0]	ledOut		;

	reg	[7:0] regaddr		;
	wire 	[7:0] ADDR00		; //read flag
	wire 	[7:0] ADDR01		; //manual write flag 
	wire 	[7:0] ADDR02		; //manual write data
	wire 	[7:0] ADDR03		; //manual write RA
	wire 	[7:0] ADDR04		; //manual write CA
	wire 	[7:0] ADDR05		; //DAQ Start Signal
	wire 	[7:0] ADDR06		; //Trigger interval
	wire 	[7:0] ADDR07		; //CLK (shift registor) interval
	reg	[7:0] ADDR [0:7]	;	

	always@ (posedge CLK25M)begin
		if(~RSTn)begin
			regaddr <= 8'b0;
		end
		regaddr <= DIP_SW;
		ADDR[0] <= ADDR00;
		ADDR[1] <= ADDR01;
		ADDR[2] <= ADDR02;
		ADDR[3] <= ADDR03;
		ADDR[4] <= ADDR04;
		ADDR[5] <= ADDR05;
		ADDR[6] <= ADDR06;
		ADDR[7] <= ADDR07;
		case(regaddr)
			8'h0:	ledOut <= ADDR[0];
			8'h1:	ledOut <= ADDR[1];
			8'h2:	ledOut <= ADDR[2];
			8'h3:	ledOut <= ADDR[3];
			8'h4:	ledOut <= ADDR[4];
			8'h5:	ledOut <= ADDR[5];
			//8'h6:	ledOut <= ADDR[6];
			//8'h7:	ledOut <= ADDR[7];
			default:	ledOut <= 8'b0;
		endcase
	end		
	assign LED[6:0] = ledOut[6:0];
	reg orLED7;
	assign LED[7] = orLED7;

	wire [7:0] wire_addr2;
	wire [7:0] wire_addr3;
	wire [7:0] wire_addr4;
	wire [7:0] wire_addr6;
	wire [7:0] wire_addr7;
	assign wire_addr2 = ADDR[2];
	assign wire_addr3 = ADDR[3];
	assign wire_addr4 = ADDR[4];
	assign wire_addr6 = ADDR[6];
	assign wire_addr7 = ADDR[7];
//------------------------------------------------------------------------------
//SBPIX1
	reg				irUsrActive		;
	reg				irUsrCloseReq	;
	reg				irUsrTxAfull	;
	reg				irUsrRxEmpty	;
	reg				irUsrRxRv		;
	reg		[7:0]	irUsrRxRd		;

	always@ (posedge CLK25M) begin
		irUsrActive		<= USR_ACTIVE;
		irUsrCloseReq	<= USR_CLOSE_REQ;
		irUsrTxAfull	<= USR_TX_AFULL;
		irUsrRxEmpty	<= USR_RX_EMPTY;
		irUsrRxRv		<= USR_RX_RV;
		irUsrRxRd[7:0]	<= USR_RX_RD[7:0];
	end
	
	reg orCSR;
	reg orWR;
	reg orRAEN;
	reg orCAEN;
	reg [5:0] orRA;
	reg [5:0] orCA;
	reg [4:0] orDI;
	
	reg write_all;
	reg read_all;
	//reg manual_read;
	reg manual_write;
	reg stop_read_all;
	reg stop_manual_write;
	reg [5:0] write_cntRA;
	reg [5:0] write_cntCA;
	reg [3:0] write_cycle_cnt;
	reg [3:0] read_cycle_cnt;
	
	reg TCP_DATA_DO;
	
	reg DAQ_ready;

	integer pix_size;	

	always @(posedge CLK25M)begin
		if(USR_ACTIVE==1'b0)begin
			orCSR <= 1'b0;
			orWR <= 1'b0;
			orRAEN <= 1'b0;
			orCAEN <= 1'b0;
			orRA <= 6'b0;
			orCA <= 6'b0;
			orDI <= 5'b0;
			
			write_all <= 1'b1;
			read_all <= 1'b1;
			//manual_read <= 1'b0;
			manual_write <= 1'b0;
			stop_read_all <= 1'b0;
			stop_manual_write <= 1'b0;
			write_cycle_cnt <= 3'b0;	
			read_cycle_cnt <= 4'b0;	

			TCP_DATA_DO <= 1'b0;
			
			DAQ_ready <= 1'b0;

			pix_size <= 6'd47;
			
			orDI <= 5'b00000;
			
		end else if(write_all)begin
			DAQ_ready <= 1'b0;
			orDI <= 5'b00011;
			orCSR <= 1'b1;
			write_cycle_cnt <= write_cycle_cnt + 4'b1;
			case(write_cycle_cnt)
				4'h2 : begin
					orRAEN <= 1'b1;
					orCAEN <= 1'b1;
				end
				4'h5 : begin
					orWR <= 1'b1;
				end
				4'h9 : begin
					orWR <= 1'b0;
				end
				4'hc : begin
					orRAEN <= 1'b0;
					orCAEN <= 1'b0;
				end
				4'hf : begin
					if(orRA == pix_size & orCA == pix_size) begin
						orRA <= 6'b0;
						orCA <= 6'b0;
						write_all <= 1'b0;
						write_cycle_cnt <= 4'b0;
						orRA <= 6'b0;
						orCA <= 6'b0;
						orDI <= 5'b0;
						orCSR <= 1'b0;
					end else if(orCA == pix_size)begin
						orCA <= 6'b0;
						orRA <= orRA + 6'b1;
					end else begin
						orCA <= orCA + 6'b1;
					end
				end
			endcase

		end else if(read_all)begin
			DAQ_ready <= 1'b0;
			if(~stop_read_all)begin		
				orCSR <= 1'b1;
				read_cycle_cnt <= read_cycle_cnt + 4'b1;
			end
			case(read_cycle_cnt)
				4'h4 : begin
					orRAEN <= 1'b1;
					orCAEN <= 1'b1;
				end
				4'h7 : begin
					TCP_DATA_DO <= 1'b1;
				end
				4'h8 : begin
					TCP_DATA_DO <= 1'b0;
				end
				4'ha : begin
					orRAEN <= 1'b0;
					orCAEN <= 1'b0;
				end

				4'hf : begin
					if(orRA == pix_size & orCA == pix_size) begin
						orRA <= 6'b0;
						orCA <= 6'b0;
						stop_read_all <= 1'b1;
						read_cycle_cnt <= 4'b0;
						orRA <= 6'b0;
						orCA <= 6'b0;
						orCSR <= 1'b0;
					end else if(orCA == pix_size)begin
						orRA <= orRA + 6'b1;
						orCA <= 6'b0;
					end else begin
						orCA <= orCA + 6'b1;
					end
				end
			endcase
			
		/*end else if(manual_read)begin
			DAQ_ready <= 1'b0;
			read_cycle_cnt <= read_cycle_cnt + 4'b1;		
			orCSR <= 1'b1;
			orRA <= wire_addr3[5:0];
			orCA <= wire_addr4[5:0];			
			case(read_cycle_cnt)
				4'h4 : begin
					orRAEN <= 1'b1;
					orCAEN <= 1'b1;
				end
				4'h7 : begin
					TCP_DATA_DO <= 1'b1;
				end
				4'h8 : begin
					TCP_DATA_DO <= 1'b0;
				end
				4'ha : begin
					orRAEN <= 1'b0;
					orCAEN <= 1'b0;
				end
				4'hf : begin
					//write_cntRA <= 6'b0;
					//write_cntCA <= 6'b0;
					write_all <= 1'b0;
					manual_read <= 1'b0;
					stop_manual_write <= 1'b1;
					read_cycle_cnt <= 3'b0;
					orRA <= 6'b0;
					orCA <= 6'b0;
					orCSR <= 1'b0;
				end
			endcase*/
			
		end else if(manual_write)begin
			DAQ_ready <= 1'b0;
			if(~stop_manual_write)begin
				orCSR <= 1'b1;
				write_cycle_cnt <= write_cycle_cnt + 4'b1;		
				orRA <= wire_addr3[5:0];
				orCA <= wire_addr4[5:0];			
				orDI <= wire_addr2[4:0];
			end
			case(write_cycle_cnt)
				4'h2 : begin
					orRAEN <= 1'b1;
					orCAEN <= 1'b1;
				end
				4'h5 : begin
					orWR <= 1'b1;
				end
				4'h9 : begin
					orWR <= 1'b0;
				end
				4'hc : begin
					orRAEN <= 1'b0;
					orCAEN <= 1'b0;
				end
				4'hf : begin
					stop_manual_write <= 1'b1;
					write_cycle_cnt <= 4'b0;
					orRA <= 6'b0;
					orCA <= 6'b0;
					orDI <= 5'b0;
				end
			endcase
		end else begin
			DAQ_ready <= 1'b1;
		end

		if(ADDR[0] == 8'b1 & ~stop_read_all)begin
			read_all <= 1'b1;
		end else if(ADDR[0] == 8'b0)begin
			stop_read_all <= 1'b0;
			read_all <= 1'b0;
		end else begin
			read_all <= 1'b0;
		end

		if(ADDR[1] == 8'b1 & ~stop_manual_write)begin
			manual_write <= 1'b1;
		end else if(ADDR[1] == 8'b0)begin
			stop_manual_write <= 1'b0;
			manual_write <= 1'b0;
		end else begin
			manual_write <= 1'b0;
		end
		
		orLED7 <= DAQ_ready;
	end
	
	
	reg orCLK;
	reg [23:0] clk_cnt;
	reg [23:0] clk_interval;
	reg orTrig;
	reg [23:0] trig_cnt;
	reg [23:0] trig_interval;
	reg [7:0] send_cnt;
	reg TCP_DATA_DD;

	reg DAQ_orRAEN;
	reg DAQ_orCAEN;
	reg [5:0] DAQ_orRA;
	reg [5:4] DAQ_orCA;

	// CLK for the shift register
	always @(posedge CLK25M)begin
		if(~RSTn) begin
			orCLK <= 1'b0;
			clk_cnt <= 23'b0;
			clk_interval <= 23'b0;
		end else begin
			clk_interval <= ADDR[7] - 8'b1;
			if(clk_cnt == clk_interval)begin
				orCLK <= 1'b1;
				clk_cnt <= 23'b0;
			end else begin
				orCLK <= 1'b0;
				clk_cnt <= clk_cnt + 23'b1;
			end
		end
	end

	// Trigger
	always @(posedge CLK25M)begin
		if(~DAQ_ready)begin
			orTrig <= 1'b0;
			trig_cnt <= 24'b0;
			trig_interval <= 24'd15000000 - 24'b1; // 600 milli sec.
		end else if(ADDR[5] == 8'b1)begin
			//trig_interval[23:16] <= wire_addr6[7:0];
			case(ADDR[6])
				8'b0000_0010 : trig_interval <= 24'd5 - 24'b1; // 200 nano sec.
				8'b0000_0100 : trig_interval <= 24'd10 - 24'b1; // 400 nano sec.
				8'b0000_0110 : trig_interval <= 24'd15 - 24'b1; // 600 nano sec.
				8'b0000_1000 : trig_interval <= 24'd20 - 24'b1; // 800 nano sec.
				
				8'b0001_0001 : trig_interval <= 24'd25 - 24'b1; // 1 micro sec.
				8'b0001_0010 : trig_interval <= 24'd50 - 24'b1; // 2 micro sec.
				8'b0001_0100 : trig_interval <= 24'd100 - 24'b1; // 4 micro sec.
				8'b0001_0110 : trig_interval <= 24'd150 - 24'b1; // 6 micro sec.
				8'b0001_1000 : trig_interval <= 24'd200 - 24'b1; // 8 micro sec.
				
				8'b0010_0001 : trig_interval <= 24'd250 - 24'b1; // 10 micro sec.
				8'b0010_0010 : trig_interval <= 24'd500 - 24'b1; // 20 micro sec.
				8'b0010_0100 : trig_interval <= 24'd1000 - 24'b1; // 40 micro sec.
				8'b0010_0110 : trig_interval <= 24'd1500 - 24'b1; // 60 micro sec.
				8'b0010_1000 : trig_interval <= 24'd2000 - 24'b1; // 80 micro sec.
				
				8'b0011_0001 : trig_interval <= 24'd2500 - 24'b1; // 100 micro sec.
				8'b0011_0010 : trig_interval <= 24'd5000 - 24'b1; // 200 micro sec.
				8'b0011_0100 : trig_interval <= 24'd10000 - 24'b1; // 400 micro sec.
				8'b0011_0110 : trig_interval <= 24'd15000 - 24'b1; // 600 micro sec.
				8'b0011_1000 : trig_interval <= 24'd20000 - 24'b1; // 800 micro sec.
				
				8'b0100_0001 : trig_interval <= 24'd25000 - 24'b1; // 1 milli sec.
				8'b0100_0010 : trig_interval <= 24'd50000 - 24'b1; // 2 milli sec.
				8'b0100_0100 : trig_interval <= 24'd100000 - 24'b1; // 4 milli sec.
				8'b0100_0110 : trig_interval <= 24'd150000 - 24'b1; // 6 milli sec.
				8'b0100_1000 : trig_interval <= 24'd200000 - 24'b1; // 8 milli sec.
				
				8'b0101_0001 : trig_interval <= 24'd250000 - 24'b1; // 10 milli sec.
				8'b0101_0010 : trig_interval <= 24'd500000 - 24'b1; // 20 milli sec.
				8'b0101_0100 : trig_interval <= 24'd1000000 - 24'b1; // 40 milli sec.
				8'b0101_0110 : trig_interval <= 24'd1500000 - 24'b1; // 60 milli sec.
				8'b0101_1000 : trig_interval <= 24'd2000000 - 24'b1; // 80 milli sec.
				
				8'b0110_0001 : trig_interval <= 24'd2500000 - 24'b1; // 100 milli sec.
				8'b0110_0010 : trig_interval <= 24'd5000000 - 24'b1; // 200 milli sec.
				8'b0110_0100 : trig_interval <= 24'd10000000 - 24'b1; // 400 milli sec.
				8'b0110_0110 : trig_interval <= 24'd15000000 - 24'b1; // 600 milli sec.
				
				default : trig_interval <= 24'd15000000 - 24'b1; // 600 milli sec.
			endcase
			if(~irUsrTxAfull)begin
				if(trig_cnt == trig_interval)begin
					orTrig <= 1'b1;
					trig_cnt <= 24'h0;
				end else begin
					orTrig <= 1'b0;
					trig_cnt <= trig_cnt + 24'b1;
				end
			end else
				orTrig <= 1'b0;
		end else begin
			orTrig <= 1'b0;
			trig_cnt <= 24'b0;
			trig_interval <= 24'd15000000 - 24'b1; // 600 milli sec.
		end
	end

	reg DAQ_flag;
   //reg stop_data_taking;
	always @(posedge CLK25M)begin
		if(~DAQ_ready)begin
			DAQ_flag <= 1'b0;
			DAQ_orRAEN <= 1'b0;
			DAQ_orCAEN <= 1'b0;
			DAQ_orRA <= 6'b0;
			DAQ_orCA <= 2'b0;
			TCP_DATA_DD <= 1'b0;
			send_cnt <= 8'b0;
			//stop_data_taking <= 1'b0;
		end else if(orTrig & ADDR[5] == 8'b1)begin
			DAQ_flag <= 1'b1;
		end else if(DAQ_flag)begin
			if(~irUsrTxAfull)begin
			//if(~irUsrTxAfull & ~stop_data_taking)begin
				case(send_cnt)
					3'd0 : begin
						DAQ_orRAEN <= 1'b1;
						DAQ_orCAEN <= 1'b1;
						send_cnt <= send_cnt +3'b1;
					end
					3'd1 : begin
						TCP_DATA_DD <= 1'b1;
						send_cnt <= send_cnt +3'b1;
					end
					3'd3 : begin
						TCP_DATA_DD <= 1'b0;
						send_cnt <= send_cnt +3'b1;
					end
					3'd4 : begin
						if(DAQ_orRA == 6'd47 & DAQ_orCA == 2'd2) begin
							DAQ_orRAEN <= 1'b0;
							DAQ_orCAEN <= 1'b0;
							DAQ_orRA <= 6'b0;
							DAQ_orCA <= 2'b0;
							send_cnt <= 3'b0;
							DAQ_flag <= 1'b0;
						end else if(DAQ_orCA == 2'd2)begin
							DAQ_orCA <= 2'b0;
							DAQ_orRA <= DAQ_orRA + 6'b1;
							send_cnt <= 3'd1;
						end else begin
							DAQ_orCA <= DAQ_orCA + 2'b1;
							send_cnt <= 3'd1;
						end
					end
					default : 
						send_cnt <= send_cnt + 3'b1;
				endcase
			end
		end
	end	
	

	assign CSR = orCSR;
	assign WR = orWR;
	assign RAEN = orRAEN | DAQ_orRAEN;
	assign CAEN = orCAEN | DAQ_orCAEN;
	assign RA = orRA | DAQ_orRA;
	assign CA = orCA | {DAQ_orCA, 4'b0};
	assign DI = orDI;

	assign CLK = orCLK;
	assign TRIG = orTrig;
	
//------------------------------------------------------------------------------
//------------------------------------------------------------------------------

	reg				usrTxWe		;
	reg		[7:0]	usrTxWd		;
	reg				usrRxRe		;

	always@ (posedge CLK25M) begin
		usrTxWe		<=  irUsrActive & ~irUsrCloseReq & (TCP_DATA_DO | TCP_DATA_DD);
		if(read_all 
		//if((read_all | manual_read) 
			   & (read_cycle_cnt == 4'h7 | read_cycle_cnt == 4'h8))begin
			usrTxWd[7:0]	<=  {3'b0, DO};
		end else if(DAQ_flag)begin
			if(send_cnt == 3'd2)
				usrTxWd[7:0]	<=  DD[7:0];
			else if(send_cnt == 3'd3)
				usrTxWd[7:0]	<=  DD[15:8];
			else
				usrTxWd[7:0]	<=  8'b0;
		end else begin
			//usrTxWd[7:0]	<=  {8'b1111_1111};
			usrTxWd[7:0]	<=  {8'b0};
		end	
//		usrTxWd[7:0]	<=  irUsrRxRd[7:0];
		usrRxRe		<=  irUsrActive & ~irUsrCloseReq & ~irUsrRxEmpty;
	end

	reg				orUsrCloseAck	;
	reg				orUsrTxWe		;
	reg		[7:0]	orUsrTxWd		;
	reg				orUsrRxRe		;

	always@ (posedge CLK25M) begin
		orUsrCloseAck	<= irUsrCloseReq;
		orUsrTxWe		<= usrTxWe;
		orUsrTxWd[7:0]	<= usrTxWd[7:0];
		orUsrRxRe		<= usrRxRe;
	end

	assign	USR_CLOSE_ACK	= orUsrCloseAck;
	assign	USR_TX_WE		= orUsrTxWe;
	assign	USR_TX_WD		= orUsrTxWd[7:0];
	assign	USR_RX_RE		= orUsrRxRe;

//------------------------------------------------------------------------------
//	Register Access
//------------------------------------------------------------------------------
	wire	[31:0]	SIO_ADDR		;
	wire	[7:0]	SIO_WD			;
	wire			SIO_WE			;
	wire			SIO_RE			;
	wire			SIO_ACK			;
	wire			SIO_RV			;
	wire	[7:0]	SIO_RD			;

	SIO_SLAVE			SIO_SLAVE(
		.RSTn						(RSTn					),	// in	: System reset
		.FILL_ADDR				(32'h0					),	// in	: Filled address for narow address-width
		// Serial I/F
		.SCK						(REG_CLK				),	// in	: Clock
		.SCS						(REG_ACT				),	// in	: Active
		.SI						(REG_DI					),	// out	: Data input
		.SO						(REG_DO					),	// in	: Data output
		// Register I/F
		.REG_ADDR				(SIO_ADDR[31:0]			),	// out	: Address[31:0]
		.REG_WD					(SIO_WD[7:0]			),	// out	: Data[7:0]
		.REG_WE					(SIO_WE					),	// out	: Write enable
		.REG_RE					(SIO_RE					),	// out	: Read enable
		.REG_ACK					(SIO_ACK				),	// in	: Access acknowledge
		.REG_RV					(SIO_RV					),	// in	: Read valid
		.REG_RD					(SIO_RD[7:0]			)	// in	: Read data[7:0]
	);

	SIO_REG				SIO_REG(
		.SCK					(REG_CLK				),	// in	: Clock
		.RSTn					(RSTn					),	// in	: System reset
		//test by hirose -> horii
		.ADDR00						(ADDR00				),
		.ADDR01						(ADDR01				),
		.ADDR02						(ADDR02				),
		.ADDR03						(ADDR03				),
		.ADDR04						(ADDR04				),
		.ADDR05						(ADDR05				),
		.ADDR06						(ADDR06				),
		.ADDR07						(ADDR07				),
		// Register I/F	
		.REG_ADDR				(SIO_ADDR[31:0]			),	// in	: Address[31:0]
		.REG_WD					(SIO_WD[7:0]			),	// in	: Data[7:0]
		.REG_WE					(SIO_WE					),	// in	: Write enable
		.REG_RE					(SIO_RE					),	// in	: Read enable
		.REG_ACK					(SIO_ACK				),	// out	: Access acknowledge
		.REG_RV					(SIO_RV					),	// out	: Read valid
		.REG_RD					(SIO_RD[7:0]			)	// out	: Read data[7:0]
	);

//------------------------------------------------------------------------------
endmodule