#include "SiTCP.h"

SiTCP::SiTCP(){
	std::cout<<"SiTCP control --> Start."<<std::endl;
};

SiTCP::~SiTCP(){};

bool SiTCP::SetIPPort(char* IpAddr, unsigned int tcp, unsigned int udp){
	sitcpIpAddr = IpAddr;
	if(tcp<=0 || 65535<=tcp || udp<=0 || 65535<=udp){
		if(tcp<=0 || 65535<=tcp){
			std::cout<<"error : invalid port : tcp = "<<tcp<<std::endl;
		}
		if(udp<=0 || 65535<=udp){
			std::cout<<"error : invalid port : udp = "<<udp<<std::endl;
		}
		exit(1);
	}
	tcpPort = tcp;
	udpPort = udp;
	puts("");
	std::cout<<"SiTCP:IP Address = "<<sitcpIpAddr<<std::endl;
	std::cout<<"      TCP Port   = "<<tcpPort<<std::endl;
	std::cout<<"      UDP Port   = "<<udpPort<<std::endl;
	puts("");
	return true;
}

bool SiTCP::CreateUDPSock(){
	std::cout<<"Create socket for RBCP...";
	udpsock = socket(AF_INET, SOCK_DGRAM, 0);
	if(udpsock < 0){
		perror("UDP socket");
		std::cout<<"errno = "<<errno<<std::endl;
		//    return false;
		exit(1);
	}
	memset(&udpAddr, 0, sizeof(udpAddr));
	udpAddr.sin_family      = AF_INET;
	udpAddr.sin_port        = htons(udpPort);
	udpAddr.sin_addr.s_addr = inet_addr(sitcpIpAddr);
	std::cout<<"  Done"<<std::endl;
	return true;
}

bool SiTCP::CloseUDPSock(){
	std::cout<<"Close UDP Socket...";
	close(udpsock);
	std::cout<<"  Done"<<std::endl;
	return true;
}

bool SiTCP::CreateTCPSock(){
	std::cout<<"Create socket for TCP...";
	tcpsock = socket(AF_INET, SOCK_STREAM, 0);
	if(tcpsock < 0){
		perror("TCP socket");
		std::cout<<"errno = "<<errno<<std::endl;
		exit(1);
	}
	memset(&tcpAddr, 0, sizeof(tcpAddr));
	tcpAddr.sin_family      = AF_INET;
	tcpAddr.sin_port        = htons(tcpPort);
	tcpAddr.sin_addr.s_addr = inet_addr(sitcpIpAddr);
	std::cout<<"  Done"<<std::endl;

	std::cout<<"  ->Trying to connect to "<<sitcpIpAddr<<" ..."<<std::endl;
	if(connect(tcpsock, (struct sockaddr *)&tcpAddr, sizeof(tcpAddr)) < 0){
		if(errno != EINPROGRESS) perror("TCP connection");
		FD_ZERO(&rmask);
		FD_SET(tcpsock, &rmask);
		wmask = rmask;
		timeout.tv_sec = 3;
		timeout.tv_usec = 0;
    
		int rc = select(tcpsock+1, &rmask, NULL, NULL, &timeout);
		if(rc<0) perror("connect-select error");
		if(rc==0){
			puts("\n     =====time out !=====\n ");
			exit(1);
		}
		//    if(rc>0){puts("in");}
		else{
			puts("\n     ===connection error===\n ");
			exit(1);
		}
	}  
	FD_ZERO(&readfds);
	FD_SET(tcpsock, &readfds);
	FD_SET(0, &readfds);
 
	puts("  ->Connect success!");

	return true;
}

bool SiTCP::CloseTCPSock(){
	std::cout<<"Close TCP Socket...";
	close(tcpsock);
	std::cout<<"  Done"<<std::endl;
	return true;
}

void SiTCP::RBCPskeleton(unsigned char type, unsigned char command, unsigned char id,
						 unsigned char length, unsigned char address){
	sndHeader.type=type;
	sndHeader.command=command;
	sndHeader.id=id;
	sndHeader.length=length;
	sndHeader.address=htonl(address);
	memcpy(sndBuf, &sndHeader, sizeof(sndHeader));
}

int SiTCP::rcvRBCP_ACK(int out_flag){
	fd_set setSelect;
	int rcvdBytes;
	unsigned char rcvdBuf[1024];

	//  puts("\nWait to receive the ACK packet...");

	int recvVal = 1;

	while(recvVal){

		FD_ZERO(&setSelect);
		FD_SET(udpsock, &setSelect);

		timeout.tv_sec  = 3;
		timeout.tv_usec = 0;

		if(select(udpsock+1, &setSelect, NULL, NULL, &timeout)==0){
			// time out
			recvVal = 0;
			puts("\n***** Timeout ! *****");
			return -1;
		} else {
			// receive packet
			if(FD_ISSET(udpsock,&setSelect)){
				rcvdBytes=recvfrom(udpsock, rcvdBuf, 2048, 0, NULL, NULL);
				rcvdBuf[rcvdBytes]=0;
				if(out_flag){
					puts("\n***** A packet is received ! *****.");
					puts("Received data:");
	  
					int j=0;
					for(int i=0; i<rcvdBytes; i++){
						if(j==0) {
							printf("\t[%.3x]:%.2x ",i,rcvdBuf[i]);
							j++;
						}else if(j==3){
							printf("%.2x\n",rcvdBuf[i]);
							j=0;
						}else{
							printf("%.2x ",rcvdBuf[i]);
							j++;
						}
					}
					puts("\n");
				}
				recvVal = 0;
			}
		}
	}
	return 0;
	usleep(50);
}


void SiTCP::CSR_Read(bool start){
	RBCPskeleton(0xff, 0x80, 0x1, 0x1, 0x0);

	sndData[0] = start ? 1 : 0;
	memcpy(sndBuf+sizeof(sndHeader), sndData, 1);

	int len = 0;
	len = sendto(udpsock, sndBuf, 1 + sizeof(sndHeader), 0,
				 (struct sockaddr *)&udpAddr, sizeof(udpAddr));
	if(len < 0){
		perror("Send CSR Read Signal");
		exit(1);
	}
	if(start)std::cout<<"Send CSR Read Signal to SEABAS... Done"<<std::endl;
	else std::cout<<"Send turn off Signal (CSR Read) to SEABAS... Done"<<std::endl;
	rcvRBCP_ACK(0);
	usleep(1);
}


void SiTCP::CSR_Read_all(char* filename){
	
	std::ofstream csr_file(filename);
	csr_file << "CSR values" << std::endl;
	
	int csr_output[48][48];
	for(int i=0; i<48; i++){
		for(int j=0; j<48; j++){
			csr_output[i][j] = -1;
		}
	}
	
	CSR_Read(true);
	unsigned char csr = 0x20;
	int cnt_csrr = 0;
	while(1){
		if(cnt_csrr==2304) break;
		int rlen = recv(GetTCPSock(), &csr, sizeof(csr), MSG_WAITALL);
		if(rlen < 0){
			perror("receive");
		}
		if(cnt_csrr%1 == 0){
			csr_output[cnt_csrr/48][cnt_csrr%48] = (unsigned int) csr;
		}
		cnt_csrr++;
	}
	CSR_Read(false);
	
	for(int i=0; i<48; i++){
		csr_file << "RA : " << i << "\t = ";
		for(int j=0; j<48; j++){
			csr_file << csr_output[i][j] <<" ";
		}
		csr_file << std::endl;
	}
}


void SiTCP::CSR_Manual_Write(bool start){
	RBCPskeleton(0xff, 0x80, 0x1, 0x1, 0x1);

	sndData[0] = start ? 1 : 0;
	memcpy(sndBuf+sizeof(sndHeader), sndData, 1);

	int len = 0;
	len = sendto(udpsock, sndBuf, 1 + sizeof(sndHeader), 0,
				 (struct sockaddr *)&udpAddr, sizeof(udpAddr));
	if(len < 0){
		perror("Send CSR Manual Write Signal");
		exit(1);
	}
	//if(start)std::cout<<"Send CSR Manual Write Signal to SEABAS... Done"<<std::endl;
	//else std::cout<<"Send turn off Signal (CSR Manual Write) to SEABAS... Done"<<std::endl;
	rcvRBCP_ACK(0);
	usleep(1);
}


void SiTCP::CSR_Write_Data(unsigned char hex){
	RBCPskeleton(0xff, 0x80, 0x1, 0x1, 0x2);

	sndData[0] = hex;
	memcpy(sndBuf+sizeof(sndHeader), sndData, 1);

	int len = 0;
	len = sendto(udpsock, sndBuf, 1 + sizeof(sndHeader), 0,
				 (struct sockaddr *)&udpAddr, sizeof(udpAddr));
	if(len < 0){
		perror("Send CSR Manual Write Data");
		exit(1);
	}
	std::cout<<"CSR Manual Write Data = "<<(unsigned int)hex<<std::endl;
	rcvRBCP_ACK(0);
	usleep(1);
}


void SiTCP::CSR_Write_RA(unsigned short RA){
	RBCPskeleton(0xff, 0x80, 0x1, 0x1, 0x3);

	sndData[0] = (unsigned char)RA;
	memcpy(sndBuf+sizeof(sndHeader), sndData, 1);

	int len = 0;
	len = sendto(udpsock, sndBuf, 1 + sizeof(sndHeader), 0,
				 (struct sockaddr *)&udpAddr, sizeof(udpAddr));
	if(len < 0){
		perror("Send CSR Manual Write RA");
		exit(1);
	}
	//std::cout<<"Manual Write RA = "<<RA<<std::endl;
	rcvRBCP_ACK(0);
	usleep(1);
}


void SiTCP::CSR_Write_CA(unsigned short CA){
	RBCPskeleton(0xff, 0x80, 0x1, 0x1, 0x4);

	sndData[0] = (unsigned char)CA;
	memcpy(sndBuf+sizeof(sndHeader), sndData, 1);

	int len = 0;
	len = sendto(udpsock, sndBuf, 1 + sizeof(sndHeader), 0,
				 (struct sockaddr *)&udpAddr, sizeof(udpAddr));
	if(len < 0){
		perror("Send CSR Manual Write CA");
		exit(1);
	}
	//std::cout<<"Manual Write CA = "<<CA<<std::endl;
	rcvRBCP_ACK(0);
	usleep(1);
}


void SiTCP::CSR_Write_all(unsigned short csr){
	
	CSR_Write_Data(csr);
	
	int RA, CA;
	for (RA = 0; RA < 48; RA++){
		for (CA = 0; CA < 48; CA++){
			CSR_Write_RA(RA);
			CSR_Write_CA(CA);
			CSR_Manual_Write(true);
			CSR_Manual_Write(false);
		}
	}
}


void SiTCP::DAQStart(bool start){
	RBCPskeleton(0xff, 0x80, 0x1, 0x1, 0x5);

	sndData[0] = start ? 1 : 0;
	memcpy(sndBuf+sizeof(sndHeader), sndData, 1);
  
	int len = 0;
	len = sendto(udpsock, sndBuf, 1 + sizeof(sndHeader), 0,
				 (struct sockaddr *)&udpAddr, sizeof(udpAddr));
	if(len < 0){
		perror("Send DAQ Start Signal");
		exit(1);
	}
	if(start)std::cout<<"Send DAQ Start Signal to SEABAS... Done"<<std::endl;
	else std::cout<<"Send DAQ Stop Signal to SEABAS... Done"<<std::endl;
	rcvRBCP_ACK(0);
	usleep(1);
}


void SiTCP::DAQTrig(unsigned short trig_interval){
	RBCPskeleton(0xff, 0x80, 0x1, 0x1, 0x6);

	sndData[0] = (unsigned char)trig_interval;
	memcpy(sndBuf+sizeof(sndHeader), sndData, 1);
  
	int len = 0;
	len = sendto(udpsock, sndBuf, 1 + sizeof(sndHeader), 0,
				 (struct sockaddr *)&udpAddr, sizeof(udpAddr));
	if(len < 0){
		perror("Send Trigger Interval");
		exit(1);
	}
	std::cout << "Sent value for the TRG interval." << std::endl << std::endl;
	rcvRBCP_ACK(0);
	usleep(1);
}


void SiTCP::DAQCLK(unsigned short clk_interval){            // for shift register
	RBCPskeleton(0xff, 0x80, 0x1, 0x1, 0x7);

	sndData[0] = (unsigned char)clk_interval;
	memcpy(sndBuf+sizeof(sndHeader), sndData, 1);
  
	int len = 0;
	len = sendto(udpsock, sndBuf, 1 + sizeof(sndHeader), 0,
				 (struct sockaddr *)&udpAddr, sizeof(udpAddr));
	if(len < 0){
		perror("Send CLK (shift register) Interval");
		exit(1);
	}
	std::cout << "Sent value for the CLK interval (shift register)." << std::endl << std::endl;
	rcvRBCP_ACK(0);
	usleep(1);
}


/*void SiTCP::NumOfTakeData(int num){
  if(num<0 || 256<num){
    std::cout<<"# of Taking Data should be 0-255"<<std::endl;
    exit(1);
  }
  RBCPskeleton(0xff, 0x80, 0x1, 0x1, 0x1);

  sndData[0] = num;
  memcpy(sndBuf+sizeof(sndHeader), sndData, 1);
  
  int len = 0;
  len = sendto(udpsock, sndBuf, 1 + sizeof(sndHeader), 0, (struct sockaddr *)&udpAddr, sizeof(udpAddr));
  if(len < 0){
    perror("Set #event of taking data");
    exit(1);
  }
  puts("");
  if(num==0)std::cout<<"# of Taking Data == endless"<<std::endl;
  else std::cout<<"# of Taking Data == "<<num<<std::endl;
  usleep(1);
}*/

/*int SiTCP::TakeDatAt(unsigned short *data){
  bool ret = true;
  int rlen = 0;
  rlen = recv(tcpsock, &tmp[0][0], sizeof(tmp),MSG_WAITALL);
  //rlen = recv(tcpsock, &tmp[0][0], sizeof(tmp), 0);
  //std::cout<<"receive length = "<<rlen<<std::endl;
  if(rlen < 0){
    perror("receive");
    ret = false;
    //    exit(1);
  }else if(rlen != 32768){
    std::cout<<"receive failuer : rlen = "<<rlen<<std::endl; 
  }
  memcpy(data, tmp, sizeof(tmp));
  usleep(1);
  return rlen;
}*/

/*int SiTCP::tcpselect(){
  //return 0 : There is a keyboard input .
  //return 1 : tcp receiving data ready
  //return -1 : ERROR

  memcpy(&fds, &readfds, sizeof(fd_set));
  timeout.tv_sec = 3;
  timeout.tv_usec = 0;
  //  int n = select(tcpsock+1, &fds, NULL, NULL, &timeout);
  select(tcpsock+1, &fds, NULL, NULL, NULL);
  if(FD_ISSET(tcpsock, &fds)){
    return 1;
  }else if(FD_ISSET(0, &fds)){
    return 0;
  }else{
    return -1;
  }
}*/
