#include<iostream>
#include<fstream>
#include<time.h>
#include "sbpix_gui.h"
#include <TGLayout.h>

#define IpAddr "192.168.0.16"
#define tcpport 24
#define udpport 4660

//using namespace std;

MyMainFrame::MyMainFrame( const TGWindow *p, Int_t w, Int_t h )
    : TGMainFrame( p, w, h )
{
	gROOT->SetStyle("Plain");
	gStyle->SetPalette(1);

	//f1 = new TF1("f1","-x",0,128);
	//revAxis = new TGaxis(0,0,0,128,"f1",510,"-");
  
	utime = new TTimer();  
	dtime = new TTimer();
  
	//Main horizontal frame
	MainHorzFrame = new TGHorizontalFrame(this,700,500);
	Canv = new MyCanvas(MainHorzFrame,500,500);
	Canv->CanvLayout();
	MainHorzFrame->AddFrame(Canv, new TGLayoutHints(kLHintsExpandX | kLHintsExpandY, 5,5,3,1));
 
	Opt = new MyOption(MainHorzFrame,175,500);
	Opt->OptLayout();
	MainHorzFrame->AddFrame(Opt, new TGLayoutHints(kLHintsLeft,5,5,7,4));

	Para = new MyParameters(MainHorzFrame,175,500);
	Para->ParaLayout();
	MainHorzFrame->AddFrame(Para, new TGLayoutHints(kLHintsLeft,5,5,7,4));
 
	Cont = new MyCtrl(MainHorzFrame,175,500);
	Cont->CtrlLayout();
	MainHorzFrame->AddFrame(Cont, new TGLayoutHints(kLHintsLeft,5,5,8,4));
	AddFrame(MainHorzFrame, new TGLayoutHints(kLHintsExpandX | kLHintsExpandY,2,2,2,2));
  
	//Sub frame
	SubHorzFrame = new TGHorizontalFrame(this,900,200); 
	Log =  new MylogEdit(SubHorzFrame, new TGString("Log"));
	Log->EdtLayout();
	SubHorzFrame->AddFrame(Log, new TGLayoutHints(kLHintsLeft | kLHintsExpandY ,5,5,7,7)); 

	exit = new TGTextButton(SubHorzFrame,"&Exit");
	SubHorzFrame->AddFrame(exit, new TGLayoutHints(kLHintsRight | kLHintsBottom,5,5,7,7));
	reset = new TGTextButton(SubHorzFrame,"&Reset");
	SubHorzFrame->AddFrame(reset, new TGLayoutHints(kLHintsRight | kLHintsBottom,5,5,7,7));
	AddFrame(SubHorzFrame, new TGLayoutHints(kLHintsExpandX,2,2,2,2));
  
    Initialize();
	Connection();
	
	sitcp = new SiTCP();
	sitcp -> SetIPPort(IpAddr, tcpport, udpport);
	sitcp -> CreateTCPSock();
	sitcp -> CreateUDPSock();    
	
	MapSubwindows();
    Resize( GetDefaultSize() );
    MapWindow();
}


MyMainFrame::~MyMainFrame( void )
{
    Cleanup();
}


void MyMainFrame::Connection()
{
	Canv->GetDAQIntCBox()->Connect("Selected(Int_t)", "MyMainFrame", this, "SetDAQTime()");
	Canv->GetUpdInt()->Connect("ValueSet(Long_t)", "MyMainFrame", this, "SetUpdateTime()");
	/*Opt->GetDispOpt()->Connect("Selected(Int_t)","MyMainFrame",this,"UpdateTimer()");
	 Opt->GetHistCont()->Connect("Selected(Int_t)","MyMainFrame",this,"UpdateTimer()");*/
	Opt->GetHistHi()->Connect("ValueSet(Long_t)", "MyMainFrame", this, "SetHistRange()");
	Opt->GetHistLow()->Connect("ValueSet(Long_t)", "MyMainFrame", this, "SetHistRange()");
	//Opt->GetRunMode()->Connect("Clicked(Int_t)","MyMainFrame",this,"SetRunMode()");
	Cont->GetStart()->Connect("Clicked()", "MyMainFrame", this, "DoStart()");
	Cont->GetStop()->Connect("Clicked()", "MyMainFrame", this, "DoStop()");
	Cont->GetPause()->Connect("Clicked()", "MyMainFrame", this, "DoPause()");
	//Cont->GetFDir()->Connect("Clicked()", "MyMainFrame", this, "DirFind()");
	Log->GetClrLog()->Connect("Clicked()", "MyMainFrame", this, "ClearLogText()");
	Log->GetSaveLog()->Connect("Clicked()", "MyMainFrame", this, "SaveLogText()");
	reset->Connect("Clicked()", "MyMainFrame", this, "Reset()");
	exit->Connect("Clicked()", "MyMainFrame", this, "CloseWindow()");
 
	dtime->Connect("Timeout()", "MyMainFrame", this, "DAQTimer()");  
	utime->Connect("Timeout()", "MyMainFrame", this, "UpdateTimer()");  
}


void MyMainFrame::Initialize()
{
	//Canv->GetECanvas()->GetCanvas()->SetRightMargin(0.13);
	Canv->GetECanvas()->GetCanvas()->Clear();
	Canv->GetECanvas()->GetCanvas()->Update();
	Canv->GetDAQIntCBox()->Select(24);
	Canv->GetUpdInt()->SetNumber(1.00);
	Opt->GetDispOpt()->Select(1);
	Opt->GetHistHi()->SetNumber(500);
	Opt->GetHistLow()->SetNumber(0);
	for(int i=1; i<=3; i++) Opt->GetRunMode()->GetButton(i)->SetEnabled(kFALSE); // Temporary
	Para->GetShftRegCLK()->SetNumber(25);
	Para->GetTRGInputCBox()->Select(29);
	Para->GetCSRDAC()->SetNumber(3);
	Cont->GetCalib()->SetNumber(0);
	Cont->GetCalib()->SetState(kFALSE); // Temporary
	Cont->GetNevent()->SetNumber(100);
	Cont->GetCevent()->SetNumber(0);
	Cont->GetSavDat()->SetState(kButtonUp);
	Cont->GetFEntry()->SetText("data/b2pix_data");
	Cont->GetStop()->SetEnabled(kFALSE);
	Cont->GetPause()->SetEnabled(kFALSE);
	fildirpath = getenv("PWD");
	Cont -> GetDEntry() -> SetText(GetRltPath(fildirpath));

	double f_dtime = -1;
	int ID_dtime = Canv->GetDAQIntCBox()->GetSelected();
	if		(ID_dtime ==  0) f_dtime = 0.0000002 * 1000;	else if (ID_dtime ==  1) f_dtime = 0.0000004 * 1000;	else if (ID_dtime ==  2) f_dtime = 0.0000006 * 1000;	else if (ID_dtime ==  3) f_dtime = 0.0000008 * 1000;
	else if (ID_dtime ==  4) f_dtime = 0.000001 * 1000;		else if	(ID_dtime ==  5) f_dtime = 0.000002 * 1000;		else if (ID_dtime ==  6) f_dtime = 0.000004 * 1000;		else if (ID_dtime ==  7) f_dtime = 0.000006 * 1000;		else if (ID_dtime ==  8) f_dtime = 0.000008 * 1000;
	else if (ID_dtime ==  9) f_dtime = 0.00001 * 1000;		else if	(ID_dtime == 10) f_dtime = 0.00002 * 1000;		else if (ID_dtime == 11) f_dtime = 0.00004 * 1000;		else if (ID_dtime == 12) f_dtime = 0.00006 * 1000;		else if (ID_dtime == 13) f_dtime = 0.00008 * 1000;
	else if (ID_dtime == 14) f_dtime = 0.0001 * 1000;		else if	(ID_dtime == 15) f_dtime = 0.0002 * 1000;		else if (ID_dtime == 16) f_dtime = 0.0004 * 1000;		else if (ID_dtime == 17) f_dtime = 0.0006 * 1000;		else if (ID_dtime == 18) f_dtime = 0.0008 * 1000;
	else if (ID_dtime == 19) f_dtime = 0.001 * 1000;		else if	(ID_dtime == 20) f_dtime = 0.002 * 1000;		else if (ID_dtime == 21) f_dtime = 0.004 * 1000;		else if (ID_dtime == 22) f_dtime = 0.006 * 1000;		else if (ID_dtime == 23) f_dtime = 0.008 * 1000;
	else if (ID_dtime == 24) f_dtime = 0.01 * 1000;			else if	(ID_dtime == 25) f_dtime = 0.02 * 1000;			else if (ID_dtime == 26) f_dtime = 0.04 * 1000;			else if (ID_dtime == 27) f_dtime = 0.06 * 1000;			else if (ID_dtime == 28) f_dtime = 0.08 * 1000;
	else if (ID_dtime == 29) f_dtime = 0.1 * 1000;			else if	(ID_dtime == 30) f_dtime = 0.2 * 1000;			else if (ID_dtime == 31) f_dtime = 0.4 * 1000;			else if (ID_dtime == 32) f_dtime = 0.6 * 1000;
	dtime->SetTime( f_dtime );
	
	utime->SetTime( (int)(Canv->GetUpdInt()->GetNumber() * 1000) );

	curevent = 0;
	saveflag = false;
	
	// 2-D Hist for sum of all events
	h2_sum = new TH2F("name","title",48,0,48,48,0,48);
	h2_sum->SetStats(0);
	h2_sum->GetZaxis()->SetRangeUser(0,500);
	h2_sum->SetTitle("");
	
	h2_sum->SetXTitle("Column");
	h2_sum->GetXaxis()->SetTitleOffset(-13.5);
	h2_sum->GetXaxis()->CenterTitle();
	h2_sum->GetXaxis()->SetTickLength(0);
	h2_sum->GetXaxis()->SetLabelSize(0);
	
	h2_sum->SetYTitle("Row");
	h2_sum->GetYaxis()->SetTitleOffset(1.1);
	h2_sum->GetYaxis()->CenterTitle();
	h2_sum->GetYaxis()->SetTickLength(0);
	h2_sum->GetYaxis()->SetLabelSize(0);
	
	// 2-D Hist for current event
	h2_eve = new TH2F("name","title",48,0,48,48,0,48);
	h2_eve->SetStats(0);
	h2_eve->GetZaxis()->SetRangeUser(0,500);
	h2_eve->SetTitle("");
	
	h2_eve->SetXTitle("Column");
	h2_eve->GetXaxis()->SetTitleOffset(-13.5);
	h2_eve->GetXaxis()->CenterTitle();
	h2_eve->GetXaxis()->SetTickLength(0);
	h2_eve->GetXaxis()->SetLabelSize(0);
	
	h2_eve->SetYTitle("Row");
	h2_eve->GetYaxis()->SetTitleOffset(1.1);
	h2_eve->GetYaxis()->CenterTitle();
	h2_eve->GetYaxis()->SetTickLength(0);
	h2_eve->GetYaxis()->SetLabelSize(0);
	
	SetStatus("Initialization for SOI Belle II Pixel DAQ Software.", 5);
}


void MyMainFrame::SetDAQTime()
{
	double f_dtime = -1;
	int ID_dtime = Canv->GetDAQIntCBox()->GetSelected();
	if		(ID_dtime ==  0) f_dtime = 0.0000002 * 1000;	else if (ID_dtime ==  1) f_dtime = 0.0000004 * 1000;	else if (ID_dtime ==  2) f_dtime = 0.0000006 * 1000;	else if (ID_dtime ==  3) f_dtime = 0.0000008 * 1000;
	else if (ID_dtime ==  4) f_dtime = 0.000001 * 1000;		else if	(ID_dtime ==  5) f_dtime = 0.000002 * 1000;		else if (ID_dtime ==  6) f_dtime = 0.000004 * 1000;		else if (ID_dtime ==  7) f_dtime = 0.000006 * 1000;		else if (ID_dtime ==  8) f_dtime = 0.000008 * 1000;
	else if (ID_dtime ==  9) f_dtime = 0.00001 * 1000;		else if	(ID_dtime == 10) f_dtime = 0.00002 * 1000;		else if (ID_dtime == 11) f_dtime = 0.00004 * 1000;		else if (ID_dtime == 12) f_dtime = 0.00006 * 1000;		else if (ID_dtime == 13) f_dtime = 0.00008 * 1000;
	else if (ID_dtime == 14) f_dtime = 0.0001 * 1000;		else if	(ID_dtime == 15) f_dtime = 0.0002 * 1000;		else if (ID_dtime == 16) f_dtime = 0.0004 * 1000;		else if (ID_dtime == 17) f_dtime = 0.0006 * 1000;		else if (ID_dtime == 18) f_dtime = 0.0008 * 1000;
	else if (ID_dtime == 19) f_dtime = 0.001 * 1000;		else if	(ID_dtime == 20) f_dtime = 0.002 * 1000;		else if (ID_dtime == 21) f_dtime = 0.004 * 1000;		else if (ID_dtime == 22) f_dtime = 0.006 * 1000;		else if (ID_dtime == 23) f_dtime = 0.008 * 1000;
	else if (ID_dtime == 24) f_dtime = 0.01 * 1000;			else if	(ID_dtime == 25) f_dtime = 0.02 * 1000;			else if (ID_dtime == 26) f_dtime = 0.04 * 1000;			else if (ID_dtime == 27) f_dtime = 0.06 * 1000;			else if (ID_dtime == 28) f_dtime = 0.08 * 1000;
	else if (ID_dtime == 29) f_dtime = 0.1 * 1000;			else if	(ID_dtime == 30) f_dtime = 0.2 * 1000;			else if (ID_dtime == 31) f_dtime = 0.4 * 1000;			else if (ID_dtime == 32) f_dtime = 0.6 * 1000;
	dtime->SetTime( f_dtime );
}


void MyMainFrame::SetUpdateTime()
{
	utime->SetTime( (int)(Canv->GetUpdInt()->GetNumber()*1000) );
}


void MyMainFrame::DAQTimer()
{
	//SetStatus("DAQTimer!", 5);
    curevent++;
	
	unsigned char DD[288];
	int rlen = recv(sitcp->GetTCPSock(), &DD, sizeof(DD), MSG_WAITALL);
	if(rlen < 0){
		SetStatus("  ERROR: rlen < 0", 1);
		//perror("receive");
		//return 1;
	}

	for(int i=0; i<48; i++){
		for(int j=0; j<48; j++){
			hit[i][j]=0;
		}
	}
	for(int i=0; i<2304; i++){
		hit[i/48][i%48] = ((unsigned int)(0x00ff & (DD[i/8]<<(7-i%8)))>=(unsigned int)0x0080) ? 1 : 0;
		//hit[i/48][i%48] = ((unsigned int)(0x00ff & (DD[i/8]<<(i%8)))>=(unsigned int)0x0080) ? 1 : 0;
		// which corresponds to hit[RA][CA].
		// 0x00ff = 11111111, 0x0080 = 10000000, "<<": shifts to the left.
	}
	/*for(int i=0; i<48; i++){
		for(int j=0; j<48; j++){
			cout << hit[i][j] << " ";
		}
		cout << endl;
	}
	cout << endl;*/
	
	h2_eve->Reset();
	for(int i=0; i<48; i++){
		for(int j=0; j<48; j++){
			h2_sum->Fill(j,47-i,hit[i][j]);
			//h2_sum->Fill(j,47-i,hit[i][(j-8+48)%48]);
			h2_eve->Fill(j,47-i,hit[i][j]);
		}
	}
	
	if(saveflag){
	    datatree -> Fill();
	}
	
    Cont -> GetCevent()->SetIntNumber(curevent);
	if (curevent == Cont->GetNevent()->GetNumber()) DoStop();
}


void MyMainFrame::UpdateTimer()
{
	//SetStatus("UpdateTimer!", 5);
	
	fCanvas = Canv->GetECanvas()->GetCanvas();
	fCanvas->cd();
	
	fCanvas->SetLeftMargin(0.13);
	fCanvas->SetRightMargin(0.13);
	
	if ( Opt->GetDispOpt()->GetSelected() == 0 )		h2_sum->Draw("colz");
	else if ( Opt->GetDispOpt()->GetSelected() == 1 )	h2_eve->Draw("colz");
	
	TGaxis *axisY = new TGaxis(gPad->GetUxmin(),gPad->GetUymax(),gPad->GetUxmin()-0.00001,gPad->GetUymin(),0,48,510,"");
	axisY->SetLabelOffset(-0.025);
	axisY->Draw();
	TGaxis *axisX = new TGaxis(gPad->GetUxmin(),gPad->GetUymax(),gPad->GetUxmax(),gPad->GetUymax(),0,48,510,"-");
	axisX->SetLabelOffset(-0.005);
	axisX->Draw();
	TLatex *title = new TLatex(gPad->GetUxmin()+2,gPad->GetUymin()-4,"[SOI Belle II Pixel] Hit Distribution");
	title->Draw();
	
	fCanvas->Modified();
	fCanvas->Update();
}


void MyMainFrame::SetHistRange()
{
	int Hi = (int)(Opt->GetHistHi()->GetNumber());
	int Low = (int)(Opt->GetHistLow()->GetNumber());
	if(Low<Hi){
		h2_sum -> GetZaxis()->SetRangeUser(Low, Hi);
		h2_eve -> GetZaxis()->SetRangeUser(Low, Hi);
	}else{
		Opt->GetHistHi()->SetNumber(Low);
		Opt->GetHistLow()->SetNumber(Hi);
		h2_sum -> GetZaxis()->SetRangeUser(Hi, Low);
		h2_eve -> GetZaxis()->SetRangeUser(Hi, Low);
	}
}


void MyMainFrame::DoStart()
{
	if(Cont->GetSavDat()->IsOn()){
		saveflag = true;
	
		dataname = (char *)Cont->GetFEntry()->GetText();
		int retval;
		if(strlen(dataname)==0 || !strcmp(Cont->GetFEntry()->GetText(),"data/")){
			wrMsg = new TGMsgBox(gClient->GetRoot(),this,"Warnig","Save file name is not set.",kMBIconExclamation,kMBOk,&retval);
			return;
		}
		sprintf(datafilepath, "%s/%s.root", fildirpath, dataname);
		sprintf(logfilepath, "%s/%s.log", fildirpath, dataname);
		ifstream fin;
		fin.open(logfilepath, ios::in);
		if(fin){
			fin.close();
			wrMsg = new TGMsgBox(gClient->GetRoot(),this,"Warnig","Save file name already exist. Do you overwrite it?", kMBIconQuestion,kMBYes | kMBNo, &retval);
			if(retval==2) return;
		}
		logfout = new ofstream(logfilepath, ios::trunc);
		datafile = new TFile(datafilepath, "RECREATE", "B2PIX_DATA");
		char dispstr[2048];
		sprintf(dispstr, "Save file name = %s", datafilepath);
		SetStatus(dispstr, 7);
	  
		datatree = new TTree("tr", "B2PIX Data Tree");
		datatree -> Branch("nevent", &curevent, "nevent/i");
		datatree -> Branch("srclk", &sr_clk, "nevent/D");
		datatree -> Branch("trgin", &trg_input, "nevent/D");
		datatree -> Branch("hit", hit, "hit[48][48]/I");
	/*
            - C : a character string terminated by the 0 character
            - B : an 8 bit signed integer (Char_t)
            - b : an 8 bit unsigned integer (UChar_t)
            - S : a 16 bit signed integer (Short_t)
            - s : a 16 bit unsigned integer (UShort_t)
            - I : a 32 bit signed integer (Int_t)
            - i : a 32 bit unsigned integer (UInt_t)
            - F : a 32 bit floating point (Float_t)
            - D : a 64 bit floating point (Double_t)
            - L : a 64 bit signed integer (Long64_t)
            - l : a 64 bit unsigned integer (ULong64_t)
            - O : a boolean (Bool_t)
	*/
	}
	
	csr = Para->GetCSRDAC()->GetNumber();
	if( Para->GetTestIn()->IsOn() ) csr += 16;
	if( Para->GetMaskOn()->IsOn() ) csr +=  8;
	
	for(int i=1; i<=3; i++) Opt->GetRunMode()->GetButton(i)->SetEnabled(kFALSE);
	Para->GetShftRegCLK()->SetState(kFALSE);
	Para->GetTRGInputCBox()->SetEnabled(kFALSE);
	Para->GetTestIn()->SetEnabled(kFALSE);
	Para->GetMaskOn()->SetEnabled(kFALSE);
	Para->GetCSRDAC()->SetState(kFALSE);
	Cont->GetStart()->SetEnabled(kFALSE);
	Cont->GetStop()->SetEnabled(kTRUE);
	Cont->GetPause()->SetEnabled(kTRUE);
	Cont->GetCalib()->SetState(kFALSE);
	Cont->GetNevent()->SetState(kFALSE);
	Cont->GetFEntry()->SetEnabled(kFALSE);
	Cont->GetSavDat()->SetEnabled(kFALSE);
	reset->SetEnabled(kFALSE);
	
	totevent = (int) Cont->GetNevent()->GetNumber();
	
	// Clean up the signals.
	unsigned char dump[2304];
	for (int p = 0; p < 1000; p++) recv(sitcp->GetTCPSock(), &dump, sizeof(dump), MSG_DONTWAIT);
	
	// --------- Trigger and CLK Timings -----------
	int ID_sr_clk = Para->GetShftRegCLK()->GetNumber();
	send_sr_clk = ID_sr_clk;
	sr_clk = ID_sr_clk * 0.000000040;
	
	int ID_trg_input = Para->GetTRGInputCBox()->GetSelected();
	if      ( ID_trg_input ==  0 )	{ send_trg_input =		2;	trg_input = 0.0000002;	}	else if ( ID_trg_input ==  1 )	{ send_trg_input =		4;	trg_input = 0.0000004;	}	else if ( ID_trg_input ==  2 )	{ send_trg_input =		6;	trg_input = 0.0000006;	}	else if ( ID_trg_input ==  3 )	{ send_trg_input =		8;	trg_input = 0.0000008;	}
	else if ( ID_trg_input ==  4 )	{ send_trg_input = 16 + 1;	trg_input = 0.000001;	}	else if ( ID_trg_input ==  5 )	{ send_trg_input = 16 + 2;	trg_input = 0.000002;	}	else if ( ID_trg_input ==  6 )	{ send_trg_input = 16 + 4;	trg_input = 0.000004;	}	else if ( ID_trg_input ==  7 )	{ send_trg_input = 16 + 6;	trg_input = 0.000006;	}	else if ( ID_trg_input ==  8 )	{ send_trg_input = 16 + 8;	trg_input = 0.000008;	}
	else if ( ID_trg_input ==  9 )	{ send_trg_input = 32 + 1;	trg_input = 0.00001;	}	else if ( ID_trg_input == 10 )	{ send_trg_input = 32 + 2;	trg_input = 0.00002;	}	else if ( ID_trg_input == 11 )	{ send_trg_input = 32 + 4;	trg_input = 0.00004;	}	else if ( ID_trg_input == 12 )	{ send_trg_input = 32 + 6;	trg_input = 0.00006;	}	else if ( ID_trg_input == 13 )	{ send_trg_input = 32 + 8;	trg_input = 0.00008;	}
	else if ( ID_trg_input == 14 )	{ send_trg_input = 48 + 1;	trg_input = 0.0001;		}	else if ( ID_trg_input == 15 )	{ send_trg_input = 48 + 2;	trg_input = 0.0002;		}	else if ( ID_trg_input == 16 )	{ send_trg_input = 48 + 4;	trg_input = 0.0004;		}	else if ( ID_trg_input == 17 )	{ send_trg_input = 48 + 6;	trg_input = 0.0006;		}	else if ( ID_trg_input == 18 )	{ send_trg_input = 48 + 8;	trg_input = 0.0008;		}
	else if ( ID_trg_input == 19 )	{ send_trg_input = 64 + 1;	trg_input = 0.001;		}	else if ( ID_trg_input == 20 )	{ send_trg_input = 64 + 2;	trg_input = 0.002;		}	else if ( ID_trg_input == 21 )	{ send_trg_input = 64 + 4;	trg_input = 0.004;		}	else if ( ID_trg_input == 22 )	{ send_trg_input = 64 + 6;	trg_input = 0.006;		}	else if ( ID_trg_input == 23 )	{ send_trg_input = 64 + 8;	trg_input = 0.008;		}
	else if ( ID_trg_input == 24 )	{ send_trg_input = 80 + 1;	trg_input = 0.01;		}	else if ( ID_trg_input == 25 )	{ send_trg_input = 80 + 2;	trg_input = 0.02;		}	else if ( ID_trg_input == 26 )	{ send_trg_input = 80 + 4;	trg_input = 0.04;		}	else if ( ID_trg_input == 27 )	{ send_trg_input = 80 + 6;	trg_input = 0.06;		}	else if ( ID_trg_input == 28 )	{ send_trg_input = 80 + 8;	trg_input = 0.08;		}
	else if ( ID_trg_input == 29 )	{ send_trg_input = 96 + 1;	trg_input = 0.1;		}	else if ( ID_trg_input == 30 )	{ send_trg_input = 96 + 2;	trg_input = 0.2;		}	else if ( ID_trg_input == 31 )	{ send_trg_input = 96 + 4;	trg_input = 0.4;		}	else if ( ID_trg_input == 32 )	{ send_trg_input = 96 + 6;	trg_input = 0.6;		}
	
	//SetStatus("Setting CLK (shift register) and TRG intervals.", 5);
	sitcp->DAQCLK(send_sr_clk);
	sitcp->DAQTrig(send_trg_input);
	// ---------------------------------------------
	
	// --------- CSR ---------
	//SetStatus("Setting CSR values.", 5);
	sitcp->CSR_Write_all(csr);
	char ch_csr[2048];
	sprintf(ch_csr, "All CSR values are set to %i.", csr);
	SetStatus(ch_csr, 5);
	sitcp->CSR_Read_all("data/csrinit.dat"); // Read the values memorized in CSR.
	// -----------------------

	sitcp->DAQStart(true);
	if(saveflag) SetStatus("DAQ started.", 7); else SetStatus("DAQ started.", 5);
	
	dtime->TurnOn();
	utime->TurnOn();
}


void MyMainFrame::DoStop()
{
	sitcp->DAQStart(false);
	utime->TurnOff();
	dtime->TurnOff();
	
	if(saveflag) SetStatus("DAQ stopped.", 7); else SetStatus("DAQ stopped.", 5);
	char ch_nevents[800];
	sprintf(ch_nevents, "%i events were taken.", curevent);
	if(saveflag) SetStatus(ch_nevents, 7); else SetStatus(ch_nevents, 5);
	
	char ch_sr_clk[800];
	sprintf(ch_sr_clk, "CLK (shift register) interval = %e sec.", sr_clk);
	if(saveflag) SetStatus(ch_sr_clk, 7); else SetStatus(ch_sr_clk, 5);
	char ch_trg_input[800];
	sprintf(ch_trg_input, "TRG input interval = %e sec.", trg_input);
	if(saveflag) SetStatus(ch_trg_input, 7); else SetStatus(ch_trg_input, 5);
	
	if(saveflag){
		datatree -> Write();
		datafile -> Close();
		fCanvas->Print("data/sum.eps");
		saveflag = false;
		SetStatus("File saved.", 7);
	}

	// Clean up the signals from data taking.
	unsigned char DD[2304];
	for (int p = 0; p < 1000; p++) recv(sitcp->GetTCPSock(), &DD, sizeof(DD), MSG_DONTWAIT);
	
	sitcp->CSR_Read_all("data/csr.dat"); // Read the values memorized in CSR.
	SetStatus("The values for CSR are saved in the file data/csr.dat.", 5);
	
	curevent = 0;
	
	//for(int i=1; i<=3; i++) Opt->GetRunMode()->GetButton(i)->SetEnabled(kTRUE);
	for(int i=1; i<=3; i++) Opt->GetRunMode()->GetButton(i)->SetEnabled(kFALSE); // Temporary
	Para->GetShftRegCLK()->SetState(kTRUE);
	Para->GetTRGInputCBox()->SetEnabled(kTRUE);
	Para->GetTestIn()->SetEnabled(kTRUE);
	Para->GetMaskOn()->SetEnabled(kTRUE);
	Para->GetCSRDAC()->SetState(kTRUE);
	Cont->GetStart()->SetEnabled(kTRUE);
	Cont->GetStop()->SetEnabled(kFALSE);
	Cont->GetPause()->SetEnabled(kFALSE);
	//Cont->GetCalib()->SetState(kTRUE);
	Cont->GetCalib()->SetState(kFALSE); // Temporary.
	Cont->GetNevent()->SetState(kTRUE);
	Cont->GetFEntry()->SetEnabled(kTRUE);
	Cont->GetSavDat()->SetEnabled(kTRUE);
	reset->SetEnabled(kTRUE);
}


void MyMainFrame::DoPause()
{
	if(saveflag) SetStatus("DAQ paused.", 7); else SetStatus("DAQ paused.", 5);
	sitcp->DAQStart(false);
	utime->TurnOff();
	dtime->TurnOff();

	//Para->GetShftRegCLK()->SetState(kTRUE);
	//Para->GetTRGInputCBox()->SetEnabled(kTRUE);
	Cont->GetStart()->SetEnabled(kTRUE);
	Cont->GetPause()->SetEnabled(kFALSE);
}


void MyMainFrame::ClearLogText()
{
	int retval;
	wrMsg = new TGMsgBox(gClient->GetRoot(),this,"Question","Do you realy clear log?",kMBIconQuestion,kMBYes | kMBNo,&retval);

	if(retval==1) Log->GetlogEdit()->Clear();
}


void MyMainFrame::SaveLogText()
{
	SetStatus("Save log as text file", 7);
	Log->GetlogEdit()->SaveFile(0,kTRUE);
}


char *MyMainFrame::GetRltPath(char *fullpath)
{
	char scnchr[1024], pathtmp[1024];
	char *chrfm = "%s";
	char *rltvpathtmp;
	rltvpathtmp = (char *)malloc(1024);
	sprintf(scnchr, "%s/%s", getenv("HOME"), chrfm);
	if(sscanf(fullpath, scnchr, pathtmp)==1){
		sprintf(rltvpathtmp, "~/%s", pathtmp);
		return rltvpathtmp;
	}else{
		sprintf(rltvpathtmp, "~/");
		return rltvpathtmp;
	}
}


void MyMainFrame::Reset()
{
	SetStatus("Reset.", 5);
	Initialize();
}


void MyMainFrame::CloseWindow()
{
	int retval;
	clMsg = new TGMsgBox(gClient->GetRoot(),this,"Question","Are you sure to quit?",kMBIconQuestion,kMBYes | kMBNo,&retval);

	if(retval == 1){
		/*  sitcp->SetRSTInterbal(0);
		 sitcp->SetNIM_CLK(0);
		 sitcp->SetRSTHiTime(0);
		 sitcp->SetPixScanTime(0);
		 sitcp->DAC_cnt(0,0);*/
		sitcp->CloseUDPSock();
		sitcp->CloseTCPSock();
		gApplication->Terminate(0);  
	}
}


// ----------------
//   Canvas Frame
// ----------------

MyCanvas::MyCanvas(const TGWindow* p, UInt_t w, UInt_t h) : TGVerticalFrame(p,w,h)
{ 
	ECanvas = new TRootEmbeddedCanvas("Ecanvas",this,400,400);
  
	CanvOptFrame = new TGHorizontalFrame(this,500,100);  
	TimerFrame = new TGGroupFrame(CanvOptFrame,"Timer",kHorizontalFrame);
	TimerDAQFrame = new TGHorizontalFrame(TimerFrame,200,20);
	TimerUpdFrame = new TGHorizontalFrame(TimerFrame,200,20);
	DAQIntLabel = new TGLabel(TimerDAQFrame,"DAQ interval");
	DAQIntCBox = new TGComboBox(TimerDAQFrame,10);
	UpdIntLabel = new TGLabel(TimerUpdFrame,"Update interval [s]");
	UpdIntEntry = new TGNumberEntry(TimerUpdFrame, 0.1, 5, 14, TGNumberFormat::kNESRealThree, TGNumberFormat::kNEANonNegative, TGNumberFormat::kNELLimitMinMax, 0, 10);
}

void MyCanvas::CanvLayout()
{
	DAQIntCBox->AddEntry("200 nsec", 0);	DAQIntCBox->AddEntry("400 nsec", 1);	DAQIntCBox->AddEntry("600 nsec", 2);	DAQIntCBox->AddEntry("800 nsec", 3);
	DAQIntCBox->AddEntry("1 usec", 4);		DAQIntCBox->AddEntry("2 usec", 5);		DAQIntCBox->AddEntry("4 usec", 6);		DAQIntCBox->AddEntry("6 usec", 7);		DAQIntCBox->AddEntry("8 usec", 8);
	DAQIntCBox->AddEntry("10 usec", 9);		DAQIntCBox->AddEntry("20 usec", 10);	DAQIntCBox->AddEntry("40 usec", 11);	DAQIntCBox->AddEntry("60 usec", 12);	DAQIntCBox->AddEntry("80 usec", 13);
	DAQIntCBox->AddEntry("100 usec", 14);	DAQIntCBox->AddEntry("200 usec", 15);	DAQIntCBox->AddEntry("400 usec", 16);	DAQIntCBox->AddEntry("600 usec", 17);	DAQIntCBox->AddEntry("800 usec", 18);
	DAQIntCBox->AddEntry("1 msec", 19);		DAQIntCBox->AddEntry("2 msec", 20);		DAQIntCBox->AddEntry("4 msec", 21);		DAQIntCBox->AddEntry("6 msec", 22);		DAQIntCBox->AddEntry("8 msec", 23);
	DAQIntCBox->AddEntry("10 msec", 24);	DAQIntCBox->AddEntry("20 msec", 25);	DAQIntCBox->AddEntry("40 msec", 26);	DAQIntCBox->AddEntry("60 msec", 27);	DAQIntCBox->AddEntry("80 msec", 28);
	DAQIntCBox->AddEntry("100 msec", 29);	DAQIntCBox->AddEntry("200 msec", 30);	DAQIntCBox->AddEntry("400 msec", 31);	DAQIntCBox->AddEntry("600 msec", 32);
	DAQIntCBox->Select(24);
	DAQIntCBox->Resize(80, 20);
  
	AddFrame(ECanvas, new TGLayoutHints(kLHintsExpandX | kLHintsExpandY, 5,5,5,3));
  
	TimerDAQFrame->AddFrame(DAQIntLabel, new TGLayoutHints(kLHintsNormal,15,5,3,0));
	TimerDAQFrame->AddFrame(DAQIntCBox, new TGLayoutHints(kLHintsRight,4,5,1,0));
	TimerUpdFrame->AddFrame(UpdIntLabel, new TGLayoutHints(kLHintsNormal,15,5,3,0));
	TimerUpdFrame->AddFrame(UpdIntEntry, new TGLayoutHints(kLHintsRight,4,5,1,0));  
	TimerFrame->AddFrame(TimerDAQFrame, new TGLayoutHints(kLHintsCenterX, 5,5,0,0));
	TimerFrame->AddFrame(TimerUpdFrame, new TGLayoutHints(kLHintsCenterX, 5,5,0,0));
	CanvOptFrame->AddFrame(TimerFrame, new TGLayoutHints(kLHintsLeft, 5,5,0,0));
 
	AddFrame(CanvOptFrame, new TGLayoutHints(kLHintsLeft, 5,5,0,1)); 
}

MyCanvas::~MyCanvas()
{
	Cleanup();
}


// ----------------
//   Option Frame
// ----------------

MyOption::MyOption(const TGWindow* p, UInt_t w, UInt_t h) : TGVerticalFrame(p,w,h,kFixedWidth)
{
	DOptFrame = new TGGroupFrame(this,"Display",kVerticalFrame);
	DOptLabel = new TGLabel(DOptFrame,"\nHistogram");
	DOptCBox = new TGComboBox(DOptFrame,100);
	HistRangeLabel = new TGLabel(DOptFrame,"\nRange");
	HistRangeLFrame = new TGHorizontalFrame(DOptFrame,100,100);
	HistRangeEFrame = new TGHorizontalFrame(DOptFrame,100,100);
	RangeHiLabel = new TGLabel(HistRangeLFrame,"         Max");
	RangeLowLabel = new TGLabel(HistRangeLFrame,"Min");
	HistHiEntry = new TGNumberEntry(HistRangeEFrame, 0.1, 5, 14, TGNumberFormat::kNESInteger, TGNumberFormat::kNEANonNegative, TGNumberFormat::kNELLimitMinMax, 0, 4096);
	HistLowEntry = new TGNumberEntry(HistRangeEFrame, 0.1, 5, 14, TGNumberFormat::kNESInteger, TGNumberFormat::kNEANonNegative, TGNumberFormat::kNELLimitMinMax, 0, 4096);

	RunVertical = new TGVerticalFrame(this, 100, 100);
	TakePedeRadio = new TGButtonGroup(RunVertical, "Run Mode", kVerticalFrame);
	fR[0] = new TGRadioButton(TakePedeRadio, new TGHotString("&Calib Data Only"));
	fR[1] = new TGRadioButton(TakePedeRadio, new TGHotString("&Data Only"));
	fR[2] = new TGRadioButton(TakePedeRadio, new TGHotString("&Calib Data & Data"));
}

void MyOption::OptLayout()
{
	DOptCBox->AddEntry("Sum of events", 0);
	DOptCBox->AddEntry("Current event", 1);
	DOptCBox->Select(1);
	DOptCBox->Resize(150, 20);
	DOptFrame->AddFrame(DOptLabel, new TGLayoutHints(kLHintsLeft,5,5,3,1));
	DOptFrame->AddFrame(DOptCBox, new TGLayoutHints(kLHintsExpandX,5,5,3,1));
	DOptFrame->AddFrame(HistRangeLabel, new TGLayoutHints(kLHintsLeft,5,5,3,1));
	DOptFrame->AddFrame(HistRangeLFrame, new TGLayoutHints(kLHintsLeft,5,5,0,0));
	DOptFrame->AddFrame(HistRangeEFrame, new TGLayoutHints(kLHintsLeft,5,5,0,0));
	
	HistRangeLFrame->AddFrame(RangeHiLabel, new TGLayoutHints(kLHintsRight,4,3,0,0));  
	HistRangeLFrame->AddFrame(RangeLowLabel, new TGLayoutHints(kLHintsRight,4,3,0,0));  
	HistRangeEFrame->AddFrame(HistHiEntry, new TGLayoutHints(kLHintsRight,4,5,1,0));  
	HistRangeEFrame->AddFrame(HistLowEntry, new TGLayoutHints(kLHintsRight,4,5,1,0));
 
	fR[1]->SetState(kButtonDown);
	TakePedeRadio->Show();
	//  RunVertical->AddFrame(TakePedeRadio, new TGLayoutHints(kLHintsLeft,0,0,0,0));
	RunVertical->AddFrame(TakePedeRadio, new TGLayoutHints(kLHintsExpandX,0,0,0,0));

	AddFrame(DOptFrame, new TGLayoutHints(kLHintsExpandX,5,5,0,1));
	AddFrame(RunVertical, new TGLayoutHints(kLHintsExpandX,5,5,0,0));
}

MyOption::~MyOption()
{
	Cleanup();
}


// ----------------------------
//   Pixel Parameters Setting
// ----------------------------

MyParameters::MyParameters(const TGWindow* p, UInt_t w, UInt_t h) : TGVerticalFrame(p,w,h,kFixedWidth)
{
	PIXParaFrame = new TGGroupFrame(this,"Pixel Parameters",kVerticalFrame);
	ShftRegCLKLabel = new TGLabel(PIXParaFrame,"\nCLK for shift registers\n   (Interval: x 40 nsec)");
	ShftRegCLKEntry = new TGNumberEntry(PIXParaFrame, 0.1, 5, 14,
										TGNumberFormat::kNESInteger,
										TGNumberFormat::kNEANonNegative,
										TGNumberFormat::kNELLimitMinMax,
										2, 255);
	TRGInputLabel = new TGLabel(PIXParaFrame,"\nTRG interval");
	TRGInputCBox = new TGComboBox(PIXParaFrame, 10);
	CSRLabel = new TGLabel(PIXParaFrame,"\nControl registers");
	TestIn = new TGCheckButton(this, "Enable test input.", 1);
	MaskOn = new TGCheckButton(this, "Mask trigger signal.", 1);
	CSRDACFrame = new TGHorizontalFrame(PIXParaFrame,100,100);
	CSRDACLabel = new TGLabel(CSRDACFrame,"Adjust Vth.");
	CSRDACEntry = new TGNumberEntry(CSRDACFrame, 0.1, 5, 14,
									TGNumberFormat::kNESInteger,
									TGNumberFormat::kNEANonNegative,
									TGNumberFormat::kNELLimitMinMax,
									0, 7);
}

void MyParameters::ParaLayout()
{
	TRGInputCBox->AddEntry("200 nsec", 0);	TRGInputCBox->AddEntry("400 nsec", 1);	TRGInputCBox->AddEntry("600 nsec", 2);	TRGInputCBox->AddEntry("800 nsec", 3);
	TRGInputCBox->AddEntry("1 usec", 4);	TRGInputCBox->AddEntry("2 usec", 5);	TRGInputCBox->AddEntry("4 usec", 6);	TRGInputCBox->AddEntry("6 usec", 7);	TRGInputCBox->AddEntry("8 usec", 8);
	TRGInputCBox->AddEntry("10 usec", 9);	TRGInputCBox->AddEntry("20 usec", 10);	TRGInputCBox->AddEntry("40 usec", 11);	TRGInputCBox->AddEntry("60 usec", 12);	TRGInputCBox->AddEntry("80 usec", 13);
	TRGInputCBox->AddEntry("100 usec", 14);	TRGInputCBox->AddEntry("200 usec", 15);	TRGInputCBox->AddEntry("400 usec", 16);	TRGInputCBox->AddEntry("600 usec", 17);	TRGInputCBox->AddEntry("800 usec", 18);
	TRGInputCBox->AddEntry("1 msec", 19);	TRGInputCBox->AddEntry("2 msec", 20);	TRGInputCBox->AddEntry("4 msec", 21);	TRGInputCBox->AddEntry("6 msec", 22);	TRGInputCBox->AddEntry("8 msec", 23);
	TRGInputCBox->AddEntry("10 msec", 24);	TRGInputCBox->AddEntry("20 msec", 25);	TRGInputCBox->AddEntry("40 msec", 26);	TRGInputCBox->AddEntry("60 msec", 27);	TRGInputCBox->AddEntry("80 msec", 28);
	TRGInputCBox->AddEntry("100 msec", 29);	TRGInputCBox->AddEntry("200 msec", 30);	TRGInputCBox->AddEntry("400 msec", 31);	TRGInputCBox->AddEntry("600 msec", 32);
	TRGInputCBox->Select(29);
	TRGInputCBox->Resize(80, 20);
	
	PIXParaFrame->AddFrame(ShftRegCLKLabel, new TGLayoutHints(kLHintsLeft,0,5,3,1));
	PIXParaFrame->AddFrame(ShftRegCLKEntry, new TGLayoutHints(kLHintsExpandX,5,5,3,1));
	PIXParaFrame->AddFrame(TRGInputLabel, new TGLayoutHints(kLHintsLeft,0,5,3,1));
	PIXParaFrame->AddFrame(TRGInputCBox, new TGLayoutHints(kLHintsExpandX,5,5,3,1));
	PIXParaFrame->AddFrame(CSRLabel, new TGLayoutHints(kLHintsLeft,0,5,3,1));
	PIXParaFrame->AddFrame(TestIn, new TGLayoutHints(kLHintsLeft,10,5,3,1));
	PIXParaFrame->AddFrame(MaskOn, new TGLayoutHints(kLHintsLeft,10,5,3,1));
	CSRDACFrame->AddFrame(CSRDACEntry, new TGLayoutHints(kLHintsLeft,0,5,3,1));
	CSRDACFrame->AddFrame(CSRDACLabel, new TGLayoutHints(kLHintsLeft,5,5,3,1));
	PIXParaFrame->AddFrame(CSRDACFrame, new TGLayoutHints(kLHintsLeft,5,5,3,1));

	AddFrame(PIXParaFrame, new TGLayoutHints(kLHintsExpandX,5,5,0,1));
}

MyParameters::~MyParameters()
{
	Cleanup();
}


// -----------------
//   Control Frame
// -----------------

MyCtrl::MyCtrl(const TGWindow* p, UInt_t w, UInt_t h) : TGVerticalFrame(p,w,h,kFixedWidth)
{
	start = new TGTextButton(this,"&Start");
	stop = new TGTextButton(this,"&Stop");
	pause = new TGTextButton(this,"&Pause");
	start->SetWidth(150);
	stop->SetWidth(150);
	pause->SetWidth(150);

	CEntLabel = new TGLabel(this,"Number of Calib. events");
	NeveLabel = new TGLabel(this,"Number of events");
	CeveLabel = new TGLabel(this,"Current event");
	CalibEntry = new TGNumberEntry(this, 0, 10, 0, TGNumberFormat::kNESInteger, TGNumberFormat::kNEANonNegative, TGNumberFormat::kNELNoLimits, 0, 5000);
	NeveEntry = new TGNumberEntry(this, 0, 10, 11,TGNumberFormat::kNESInteger, TGNumberFormat::kNEANonNegative, TGNumberFormat::kNELNoLimits, 0, 80000);
	CeveEntry = new TGNumberEntryField(this, 12, 0,TGNumberFormat::kNESInteger, TGNumberFormat::kNEANonNegative, TGNumberFormat::kNELNoLimits, 0, 0);
	SavDat = new TGCheckButton(this, "save data", 1);
	SavDirEntry = new TGTextEntry(this);
	SavFilEntry = new TGTextEntry(this);
	SavDirLabel = new TGLabel(this,"Save directory path\n(relative path)");
	SavDirSel = new TGPictureButton(this,"/usr/local/root/icons/fileopen.xpm");
	SavFilLabel = new TGLabel(this,"Save file name");

	CeveEntry->SetEnabled(kFALSE);
	SavDirEntry->SetEnabled(kFALSE);
}

void MyCtrl::CtrlLayout()
{
	AddFrame(start, new TGLayoutHints(kLHintsExpandX,30,30,5,4));
	AddFrame(stop, new TGLayoutHints(kLHintsExpandX,30,30,3,4));
	AddFrame(pause, new TGLayoutHints(kLHintsExpandX,30,30,3,4));

	AddFrame(CEntLabel, new TGLayoutHints(kLHintsLeft,35,5,15,1));//left,right,top,bottom
	AddFrame(CalibEntry, new TGLayoutHints(kLHintsExpandX,44,38,1,3));
	AddFrame(NeveLabel, new TGLayoutHints(kLHintsLeft,35,5,3,1));
	AddFrame(NeveEntry, new TGLayoutHints(kLHintsExpandX,45,38,1,3));
	AddFrame(CeveLabel, new TGLayoutHints(kLHintsLeft,35,5,3,1));
	AddFrame(CeveEntry, new TGLayoutHints(kLHintsExpandX,46,52,1,3));

	AddFrame(SavDat, new TGLayoutHints(kLHintsLeft,25,5,30,1));
	AddFrame(SavDirLabel, new TGLayoutHints(kLHintsLeft,25,5,5,1));
	AddFrame(SavDirEntry, new TGLayoutHints(kLHintsExpandX,25,25,1,1));
	AddFrame(SavDirSel, new TGLayoutHints(kLHintsRight,25,25,1,1));

	AddFrame(SavFilLabel, new TGLayoutHints(kLHintsLeft,25,5,5,1));
	AddFrame(SavFilEntry, new TGLayoutHints(kLHintsExpandX,25,25,1,1));
}

MyCtrl::~MyCtrl()
{
	Cleanup();
}


// --------------
//   Log Editor
// --------------

MylogEdit::MylogEdit(const TGWindow* p, TGString* title) : TGGroupFrame(p,title,kHorizontalFrame)
{
	LogCtrlFrame = new TGVerticalFrame(this, 50, 100);
	LogEdit = new TGTextEdit(this, 600, 120);
	clearlog = new TGTextButton(LogCtrlFrame, "&Clear Log");
	savelog = new TGTextButton(LogCtrlFrame, "Save &Log");
}

void MylogEdit::EdtLayout()
{
	LogCtrlFrame->AddFrame(savelog, new TGLayoutHints(kLHintsCenterX | kLHintsBottom,2,2,3,3));
	LogCtrlFrame->AddFrame(clearlog, new TGLayoutHints(kLHintsCenterX | kLHintsBottom,2,2,3,3));

	AddFrame(LogEdit, new TGLayoutHints(kLHintsExpandX | kLHintsExpandY,2,2,3,3));  
	AddFrame(LogCtrlFrame, new TGLayoutHints(kLHintsRight | kLHintsBottom,2,2,3,3));

	LogEdit->EnableMenu(kFALSE);
	LogEdit->SetReadOnly(kTRUE); 
}

MylogEdit::~MylogEdit()
{
	Cleanup();
}


// ------------
//   Menu Bar
// ------------

MyMenuBar::MyMenuBar(const TGWindow *p) : TGMenuBar(p)
{
	MenuFile = new TGPopupMenu(gClient->GetRoot());
	MenuRun = new TGPopupMenu(gClient->GetRoot());
}

void MyMenuBar::MenuLayout()
{
	MenuFile->AddEntry("&Reset all", M_FILE_ALLRESET);
	MenuFile->AddEntry("&Save", M_FILE_SAVE);
	MenuFile->AddEntry("S&ave as...", M_FILE_SAVEAS);
	MenuFile->AddSeparator();
	MenuFile->AddEntry("E&xit", M_FILE_EXIT);

	MenuRun->AddEntry("&Start", M_RUN_START);
	MenuRun->AddEntry("S&top", M_RUN_STOP);
	MenuRun->AddEntry("&Pause", M_RUN_PAUSE);
	MenuRun->AddSeparator();
	MenuRun->AddEntry("&Reset Canvas", M_RUN_RESETCANV);
 
	AddPopup("&File", MenuFile, new TGLayoutHints(kLHintsTop | kLHintsLeft, 0, 4, 0, 0));
	AddPopup("&Run", MenuRun, new TGLayoutHints(kLHintsTop | kLHintsLeft, 0, 4, 0, 0));
}

MyMenuBar::~MyMenuBar()
{
	Cleanup();
}


// -------------------
//   Other Functions
// -------------------

void MyMainFrame::SetStatus(char *string, unsigned char flags)
{
	char dispstr[2048];
	int yr, mt, dt, hr, mn, sc;

	if (flags & 0x04){
		tstmp = new TTimeStamp(); 
		int a = tstmp->GetDate(kFALSE);
		yr = a/10000;
		mt = (a%10000)/100;
		dt = (a%10000)%100;
		int b = tstmp->GetTime(kFALSE);
		hr = b/10000;
		mn = (b%10000)/100;
		sc = (b%10000)%100;
		sprintf(dispstr,"[%4d-%02d-%02d %02d:%02d:%02d] %s",yr,mt,dt,hr,mn,sc,string);
	}else{
		strcpy(dispstr,string);
	}

	if (flags & 0x01){
		Log->GetlogEdit()->AddLine(dispstr) ;
		Log->GetlogEdit()->ScreenDown() ;
	}

	if ((flags & 0x02) && logfout){
		*logfout << dispstr << endl;
	}
}


// --------
//   Main
// --------

int main( int argc, char **argv )
{
	TApplication theApp( "app", &argc, argv );

    MyMainFrame gui( gClient->GetRoot(), 400, 300 );
	gui.SetWindowName("SOI Belle II Pixel Operation");

    theApp.Run();
    return 0;
}
