#include <TApplication.h>
#include <TGFrame.h>
#include <TGButton.h>
#include <TGButtonGroup.h>
#include <TGClient.h>
#include <TGComboBox.h>
#include <TGLabel.h>
#include <TGMenu.h>
#include <TGMsgBox.h>
#include <TGNumberEntry.h>
#include <TGTextEdit.h>
#include <TGWindow.h>
#include <TRootEmbeddedCanvas.h>
#include <TTimeStamp.h>
#include <TTimer.h>

#include "TH2.h"
#include "TCanvas.h"
#include "TStyle.h"
#include "TROOT.h"
#include "TGaxis.h"
#include "TLatex.h"
#include "TTree.h"
#include "TFile.h"

//#include <iostream>
//#include <fstream>
//#include <stdlib.h>

using namespace std;

#ifndef __CINT__
//#include <string.h>
//#include <dirent.h>
//#include "calib.h"
#include "SiTCP.h"
#endif

class MyCanvas;
class MyOption;
class MyParameters;
class MyCtrl;
class MylogEdit;
class MyMenuBar;

class MyMainFrame : public TGMainFrame
{
 private:
	MyCanvas *Canv;
	MyOption *Opt;
	MyParameters *Para;
	MyCtrl *Cont;
	MylogEdit *Log;
	TGHorizontalFrame *MainHorzFrame, *SubHorzFrame;

	TGMsgBox *wrMsg;
	TGMsgBox *clMsg;
  
	TTimeStamp *tstmp;

    TTimer *utime, *dtime;
	
    char *fildirpath, *dataname;
    char datafilepath[1024], logfilepath[1024];
    ofstream *logfout;
    TFile *datafile;
	TTree *datatree;
	
	TCanvas *fCanvas;
	int hit[48][48];
	TH2F *h2_sum, *h2_eve;
	TGaxis *axisY;
	TGaxis *axisX;
	TLatex *title;
	
    int send_sr_clk, send_trg_input;
	double sr_clk, trg_input;
	
	unsigned short csr;
	
	int totevent, curevent;
	bool saveflag;
	
    TGTextButton *reset;
    TGTextButton *exit;

#ifndef __CINT__  
	SiTCP *sitcp;
#endif
    
 public:
    MyMainFrame( const TGWindow *p, Int_t w, Int_t h );
    virtual ~MyMainFrame( void );

	void Connection();
	void Initialize();
	
    void SetDAQTime();
    void SetUpdateTime();
	void DAQTimer();
    void UpdateTimer();
	
    void SetHistRange();
  
    void DoStart();
    void DoStop();
    void DoPause();
	
	void ClearLogText();
	void SaveLogText();
	
    char *GetRltPath(char *fullpath);
  
	void Reset();
	void CloseWindow();
	
	void SetStatus(char *string, unsigned char flags);

    ClassDef( MyMainFrame, 0 );
};


class MyCanvas : public TGVerticalFrame
{
 protected:
	TGGroupFrame *TimerFrame, *AnglFrame;
	TGHorizontalFrame  *CanvOptFrame, *TimerDAQFrame, *TimerUpdFrame;
	TGLabel *DAQIntLabel, *UpdIntLabel;
	TRootEmbeddedCanvas *ECanvas;
	TGNumberEntry *UpdIntEntry;
	TGComboBox *DAQIntCBox;
 public:
	MyCanvas(const TGWindow* p, UInt_t w, UInt_t h);
	virtual ~MyCanvas();
	void CanvLayout();
	TRootEmbeddedCanvas *GetECanvas() const { return ECanvas;}
	TGNumberEntry *GetUpdInt() const { return UpdIntEntry;}
	TGComboBox *GetDAQIntCBox() const { return DAQIntCBox;} 
};


class MyOption : public TGVerticalFrame
{
 protected:
	TGHorizontalFrame *HistRangeEFrame, *HistRangeLFrame;
	TGVerticalFrame *RunVertical;
	TGGroupFrame *DOptFrame;
	TGLabel *DOptLabel, *HistRangeLabel, *RangeHiLabel, *RangeLowLabel;
	TGComboBox *DOptCBox;
	TGNumberEntry *HistHiEntry, *HistLowEntry;
	TGButtonGroup *TakePedeRadio;
	TGRadioButton *fR[3];
  
 public:
	MyOption(const TGWindow* p, UInt_t w, UInt_t h);
	virtual ~MyOption();
	void OptLayout();
	TGComboBox *GetDispOpt() const { return DOptCBox;}
	TGNumberEntry *GetHistHi() const { return HistHiEntry;}
	TGNumberEntry *GetHistLow() const { return HistLowEntry;}
	TGButtonGroup *GetRunMode() const { return TakePedeRadio;}
};


class MyParameters : public TGVerticalFrame
{
 protected:
	TGGroupFrame *PIXParaFrame;
	TGHorizontalFrame *CSRDACFrame;
	TGLabel *ShftRegCLKLabel, *TRGInputLabel, *CSRLabel, *CSRDACLabel;
	TGNumberEntry *ShftRegCLKEntry, *CSRDACEntry;
	TGComboBox *TRGInputCBox;
	TGCheckButton *TestIn, *MaskOn;
 public:
	MyParameters(const TGWindow* p, UInt_t w, UInt_t h);
	virtual ~MyParameters();
	void ParaLayout();
	TGNumberEntry *GetShftRegCLK() const { return ShftRegCLKEntry;}
	TGComboBox *GetTRGInputCBox() const { return TRGInputCBox;}
	TGCheckButton *GetTestIn() const { return TestIn;}
	TGCheckButton *GetMaskOn() const { return MaskOn;}
	TGNumberEntry *GetCSRDAC() const { return CSRDACEntry;}
};


class MyCtrl : public TGVerticalFrame
{
 protected:
	TGLabel *NeveLabel, *CeveLabel, *SavDirLabel, *SavFilLabel, *CEntLabel;
	TGTextButton *start, *stop, *pause;
	TGNumberEntry *CalibEntry;
	TGNumberEntry *NeveEntry;
	TGNumberEntryField *CeveEntry;
	TGCheckButton *SavDat;
	TGTextEntry *SavFilEntry, *SavDirEntry;
	TGPictureButton *SavDirSel;
 public:
	MyCtrl(const TGWindow* p, UInt_t w, UInt_t h);
	virtual ~MyCtrl();
	void CtrlLayout();
	TGTextButton *GetStart() const { return start;}
	TGTextButton *GetStop() const { return stop;}
	TGTextButton *GetPause() const { return pause;}
	TGNumberEntry *GetCalib() const { return CalibEntry;}
	TGNumberEntry *GetNevent() const { return NeveEntry;}
	TGNumberEntryField *GetCevent() const { return CeveEntry;}
	TGTextEntry *GetDEntry() const { return SavDirEntry;}
	TGTextEntry *GetFEntry() const { return SavFilEntry;}
	TGCheckButton *GetSavDat() const { return SavDat;}
	TGPictureButton *GetFDir() const { return SavDirSel;}
};


class MylogEdit : public TGGroupFrame
{
 protected:
	TGTextEdit *LogEdit;
	TGVerticalFrame *LogCtrlFrame;
	TGTextButton *clearlog, *savelog;
 public:
	MylogEdit(const TGWindow* p, TGString* title);
	virtual ~MylogEdit();
	void EdtLayout();
	TGTextEdit *GetlogEdit() const { return LogEdit;}
	TGTextButton *GetClrLog() const { return clearlog;}
	TGTextButton *GetSaveLog() const { return savelog;}
};


class MyMenuBar : public TGMenuBar
{
 protected:
	TGPopupMenu    *MenuFile, *MenuRun;
	TGGroupFrame *EventFrame;
 public:
	MyMenuBar(const TGWindow *p);
	virtual ~MyMenuBar();
	void MenuLayout();
};


enum MenuCommandId {
	M_FILE_ALLRESET,
	M_FILE_SAVE,
	M_FILE_SAVEAS,
	M_FILE_EXIT,

	M_RUN_START,
	M_RUN_STOP,
	M_RUN_PAUSE,
	M_RUN_RESETCANV
};
